/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */

#ifndef BLOCK_HPP
#define BLOCK_HPP

#include <limits>

//#include <array> //works in MSVC?
#include <tr1/array> //works in g++
//#include <boost/tr1/array.hpp> //works if you have Boost

typedef std::tr1::array<double,3> Vector3d;

class Block
{
	public:
		Block() {
			Vector3d v;
			v[0] = v[1] = v[2] = 1.0;
			setUp(v);
		}
		Block(Vector3d &v) {
			setUp(v);
		}
		void setUp(Vector3d &dims) {
			for(unsigned i = 0; i < 3; ++i) {
				dimensions[i] = dims[i];
				for(unsigned j = 0; j < 2; ++j) neighbours[i][j]=0;
			}
		}
		double trace(Vector3d& direction, Vector3d position=Vector3d(), double distance=0);
		void add(Block* b, unsigned dir, unsigned side) {
			neighbours[dir][side] = b;
			b->neighbours[dir][1-side] = this;
		}
		Block* neighbour(unsigned i, bool b) { return neighbours[i][b?1:0]; }
		double dim(int i) { return dimensions[i]; }
	private:
		Block* neighbours[3][2];
		Vector3d dimensions;
};

#endif
