/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */

#ifndef PLAYER_HPP
#define PLAYER_HPP

#include "Block.hpp"
#include "CGALDefines.hpp"
#include <cmath>//For sqrt,min,max
#include <utility>//pair

class Player
{
	public:
		Player(Block* b);
		Player(Block* b, Vector3d& p);
		
		//Control
		void turn(double horiz, double vert);
		void setForward(bool f) { forwardPressed = f; }
		void setBackward(bool f) { backwardPressed = f; }
		void setLeft(bool f) { leftPressed = f; }
		void setRight(bool f) { rightPressed = f; }
		
		void update();
		std::pair<double,double> look(DataStructure::Vertex_iterator, DataStructure::Vertex_iterator);
		double look(double x, double y);
	private:
		//World location
		Block* block;

		//view angles in radians (player-->world)
		double turnHoriz;
		double turnVert;
		
		//Player controlled movement
		bool forwardPressed;
		bool backwardPressed;
		bool leftPressed;
		bool rightPressed;
		
		//Turned in this frame (do we need to update viewMatrix?)
		bool turned;
		
		//For playerRadius
		Block *partialBlocks[3];
		
		Vector3d position;//world coords in cube
		double viewMatrix[3][3];

		void move(Vector3d&);
		bool freeToPass(unsigned, bool);
		void turn();
		void direct(Vector3d& v);
		//converts a vector from player coords to world coords
		
		double moveSpeed;
		double playerRadius;
		//Do not be deceived: The player is a cube, not a sphere.
		/* Also: MSVC doesn't like static const doubles initialised here,
		so they get to be instance variables instead. Hrm. */
		
		enum HitNature {
			Invalid=0, Face, EnterBoundary, ExitBoundary
		};
		
		static void normalise(Vector3d& v, double magnitude=1.0) {
			double norm = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
			v[0]/=norm;
			v[1]/=norm;
			v[2]/=norm;
			v[0]*=magnitude;
			v[1]*=magnitude;
			v[2]*=magnitude;
		}
};

#endif
